@@ -3,8 +3,11 @@ from __future__ import annotations
 from collections import defaultdict
 import inspect
 from typing import Any, DefaultDict, Dict
+import warnings
 
 from sphinx.ext.napoleon import Config, GoogleDocstring, NumpyDocstring  # type: ignore
+from typestring_parser import parse as typstr_parse                      # type: ignore
+from typestring_parser.errors import UnsupportedTypeString               # type: ignore
 
 from .errors import UnsupportedDocstringStyle
 
@@ -14,8 +17,12 @@ GOOGLE_HEADER = 'Args:'
 NUMPY_HEADER = 'Parameters\n----------'
 
 
-def parse_docstring(doc: str) -> Dict[str, Dict[str, Any]]:
-    doc = inspect.cleandoc(doc)
+def parse_docstring(obj) -> Dict[str, Dict[str, Any]]:
+    """Parse the given docstring or the given obj's docstring."""
+    if isinstance(obj, str):
+        doc, func = inspect.cleandoc(obj), None
+    else:
+        doc, func = inspect.getdoc(obj), obj  # type: ignore
     if NUMPY_HEADER in doc:
         lines = NumpyDocstring(doc, config=CONFIG).lines()
     elif GOOGLE_HEADER in doc:
@@ -30,7 +37,14 @@ def parse_docstring(doc: str) -> Dict[str, Dict[str, Any]]:
             name, parameters[name]['help'] = _find_name_and_remainder(line)
         elif line.startswith(':type'):
             name, type_string = _find_name_and_remainder(line)
-            parameters[name]['type'] = _parse_type_string(type_string)
+            try:
+                parameters[name]['type'] = typstr_parse(type_string, func=func)
+            except NameError as err:
+                _name = str(err).split("'")[1]
+                warnings.warn(f'Type hint {_name!r} cannot be resolved. '
+                               'Continuing as if no type information was provided.')
+            except UnsupportedTypeString:
+                pass
     return parameters
 
 
@@ -39,9 +53,3 @@ def _find_name_and_remainder(s):
     j = s.find(' ') + 1
     k = s.find(':', 1)
     return s[j:k], s[k+1:].lstrip()
-
-
-def _parse_type_string(s):
-    candidates = s.split(' or ')
-    candidates = [x.split(' of ')[0] for x in candidates]
-    return candidates
